/* wrapper.c - IcePick tools wrapper
   Copyright (C) 2008 Andrew John Hughes

This file is part of IcePick.

IcePick is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.
 
IcePick is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcePick; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.
*/

#include "wrapper.h"

#ifdef USE_JNI
int invoke_via_jni(char** vm_options, int vm_count,
		   char** main_options, int main_count)
{
  JNIEnv *env;
  JavaVM *vm;
  JavaVMInitArgs vm_args;
  JavaVMOption options[vm_count];
  jint res;
  jclass cls;
  jmethodID main_method;
  jclass string_class;
  jobjectArray args;
  int a;

  for (a = 0; a < vm_count; ++a)
    options[a].optionString = *vm_options++;
#ifdef DEBUG
  for (a = 0; a < vm_count; ++a)
    printf("vm option %d: %s\n", a, options[a].optionString);
#endif

  vm_args.version = JNI_VERSION_1_2;
  vm_args.ignoreUnrecognized = JNI_TRUE;
  vm_args.options = options;
  vm_args.nOptions = vm_count;

  res = JNI_CreateJavaVM(&vm, (void**)&env, &vm_args);
  if (res < 0)
    {
      fprintf(stderr, "Couldn't create Java VM\n");
      exit(-1);
    }
  cls = (*env)->FindClass(env, "CLASS_NAME");
  if (cls == NULL) {
    goto destroy;
  }
 
  main_method = (*env)->GetStaticMethodID(env, cls, "main",
					  "([Ljava/lang/String;)V");
  if (main_method == NULL) {
    goto destroy;
  }

  string_class = (*env)->FindClass(env, "java/lang/String");
  args = (*env)->NewObjectArray(env, main_count, string_class, NULL);
  if (args == NULL) {
    goto destroy;
  }

  for (a = 0; a < main_count; ++a)
    {
      jstring jstr;

      jstr = (*env)->NewStringUTF(env, *argv);
      if (jstr == NULL) 
	goto destroy;
      ++argv;
      (*env)->SetObjectArrayElement(env, args, a, jstr);
    }

  (*env)->CallStaticVoidMethod(env, cls, main_method, args);
 
 destroy:
  if ((*env)->ExceptionOccurred(env)) 
    (*env)->ExceptionDescribe(env);
  return (*vm)->DestroyJavaVM(vm);
}
#endif

int invoke_via_exec(char** vm_options, int vm_count,
		    char** main_options, int main_count)
{
  char* argv[vm_count + main_count + 5];
  char** argv_pos;
  int a;

  argv_pos = argv;
  *argv_pos++ = (char*) VM_BINARY;
  *argv_pos++ = (char*) "-classpath";
  *argv_pos++ = (char*) TOOLS_CP;
  
  for (a = 0; a < vm_count; ++a)
    *argv_pos++ = *vm_options++;

  *argv_pos++ = (char*) MAIN_METHOD;

  for (a = 0; a < main_count; ++a)
    *argv_pos++ = *main_options++;
  
#ifdef DEBUG
  for (a = 0; a < vm_count + main_count + 4; ++a)
    printf("cmd line option %d: %s\n", a, argv[a]);
#endif

  *argv_pos++ = NULL;

  int res = execvp(VM_BINARY, argv);
  if (res)
    perror("Error invoking VM");
  return res;
}

int main(int argc, char** argv)
{
  char** read_pos;
  char** write_pos;
  int a;
  int vm_count = 0;
  int main_count = 0;
  char* vm_opt[argc-1];

  read_pos = argv + 1;
  write_pos = argv;
  for (a = 0; a < argc - 1; ++a)
    {
      if (strncmp(*read_pos, "-J", (size_t)2) == 0)
	{
	  vm_opt[vm_count] = *read_pos + 2;
	  ++vm_count;
	}
      else
	{
	  *write_pos = *read_pos;
	  ++write_pos;
	  ++main_count;
	}
      ++read_pos;
    }
#ifdef DEBUG
  for (a = 0; a < main_count; ++a)
    printf("main option %d: %s\n", a, *argv++);
  for (a = 0; a < vm_count; ++a)
    printf("vm option %d: %s\n", a, vm_opt[a]);
#endif

#ifdef USE_JNI
  return invoke_via_jni(vm_opt, vm_count, argv, main_count);
#else
  return invoke_via_exec(vm_opt, vm_count, argv, main_count);
#endif

}
  
